<?php
declare(strict_types=1);

namespace Inti\Test\Constraint\Config;

use PHPUnit\Framework\Constraint\Constraint;
use function array_key_exists;
use function explode;
use function sprintf;

/**
 * Class HasKey
 *
 * @package Inti\Test\Constraint\Config
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-bartog
 */
class HasValue extends Constraint
{
    private string $value;

    public function __construct(string $value)
    {
        $this->value = $value;
    }

    protected function matches($other): bool
    {
        $keys = explode('/', $this->value);

        foreach ($keys as $key) {
            if (!array_key_exists($key, $other)) {
                return false;
            }

            $other = $other[$key];
        }

        return true;
    }

    protected function failureDescription($other): string
    {
        return 'the given config array ' . $this->toString();
    }

    public function toString(): string
    {
        return sprintf("contains the key '%s'", $this->value);
    }
}
