<?php
declare(strict_types=1);

namespace Inti\Test\Constraint\Container;

use PHPUnit\Framework\Constraint\Constraint;

use Psr\Container\ContainerInterface;
use function sprintf;

/**
 * Class ServiceInstanceOf
 *
 * @package Inti\Test\Constraint\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ServiceInstanceOf extends Constraint
{
    private string $service;
    private string $instanceOf;

    public function __construct(string $service, string $instanceOf)
    {
        $this->service = $service;
        $this->instanceOf = $instanceOf;
    }

    /**
     * @param ContainerInterface|array $other
     * @return bool
     */
    protected function matches($other): bool
    {
        $service = $other->get($this->service);
        return $service instanceof $this->instanceOf;
    }

    protected function failureDescription($other): string
    {
        return sprintf("the service '%s' %s", $this->service, $this->toString());
    }

    public function toString(): string
    {
        return sprintf("is an instance of '%s'", $this->instanceOf);
    }
}
