<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\Test\Emitter\TestEmitter;
use Laminas\Diactoros\ServerRequestFactory;
use Laminas\HttpHandlerRunner\Emitter\EmitterInterface;
use Laminas\Stdlib\ArrayUtils;
use Mezzio\Application;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

/**
 * Trait ApplicationTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
trait ApplicationTestCase
{
    use ContainerTestCase {
        ContainerTestCase::createConfig as parentCreateConfig;
    }

    protected function createConfig(array $config = []): array
    {
        return $this->parentCreateConfig(ArrayUtils::merge($config, [
            'dependencies' => [
                'services' => [
                    EmitterInterface::class => new TestEmitter(),
                ],
            ],
        ]));
    }

    protected function createApplication(array $config = []): Application
    {
        $container = $this->createContainer($config);
        return $container->get(Application::class);
    }

    protected function runApplication(
        array $config = [],
        string $path = '/',
        string $method = 'GET'
    ): ResponseInterface
    {
        $request = ServerRequestFactory::fromGlobals([
            'HTTP_HOST' => $this->hostname ?? 'inti.dev',
            'REQUEST_METHOD' => $method,
            'REQUEST_URI' => $path,
        ]);

        $config['dependencies']['services'][ServerRequestInterface::class] = fn() => $request;

        $application = $this->createApplication($config);
        $application->run();

        /** @var TestEmitter $emitter */
        $emitter = $this->container->get(EmitterInterface::class);
        return $emitter->getReponse();
    }
}
