<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\Test\Constraint\Config\HasValue;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;

use PHPUnit\Framework\Assert;

use function array_merge;

/**
 * Trait ConfigurationTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
trait ConfigTestCase
{
    protected function createConfig(array $config = []): array
    {
        $aggregator = new ConfigAggregator(array_merge($this->providers ?? [], [
            new ArrayProvider($this->config ?? []),
            new ArrayProvider($config)
        ]));

        return $aggregator->getMergedConfig();
    }

    /**
     * Provjerava da li data vrijednost postoji u datoj konfiguraciji.
     *
     * @param string $key
     * @param array $config
     * @param string $message
     */
    public function assertConfigValueExists(string $key, array $config, string $message = ''): void
    {
        $constraint = new HasValue($key);
        Assert::assertThat($config, $constraint, $message);
    }
}
